class Club < ActiveRecord::Base

  # Attributes

  # Callbacks

  # Relationships
  has_one :address, :as => :addressable, :dependent => :destroy
  has_many :contact_points, :as => :contactable, :dependent => :destroy

  # Validations
  validates_presence_of :name, :subdomain
  validates_uniqueness_of :subdomain
  validates_uniqueness_of :name

  # Custom Finders
  named_scope :for_subdomain, lambda {|s| { :conditions => { :subdomain => s } } }

  # Model Operations

  # Use this method to create a new club.  This method will
  # create a club using the required components
  def self.create_club( club_hash, address_hash, contact_point_hashes ) #membership_hashes, reservation_rule_hashes )
    club = new(
      :name => club_hash[:name],
      :subdomain => club_hash[:subdomain]
    )
    club.add_address( address_hash )
    contact_point_hashes.each { |contact_point_hash| club.add_contact_point( contact_point_hash ) }
    club
  end

  # Instance Operations

  def add_contact_point(contact_point_hash)
    self.contact_points.build(contact_point_hash)
  end

  def add_address(address_hash)
    Address.destroy( self.address ) if !self.address.nil?
    self.address= Address.new( address_hash )
  end

end
