require 'test_helper'

class AddressTest < ActiveSupport::TestCase
  fixtures :addresses
  
  def test_requires_street1
    address = Address.new
    assert(!address.valid?)
    assert(address.errors.invalid?(:street_1))
    assert(!address.save)
  end
  
  def test_requires_city
    address = Address.new
    assert(!address.valid?)
    assert(address.errors.invalid?(:city))
    assert(!address.save)
  end

  def test_requires_state_or_province
    address = Address.new
    assert(!address.valid?)
    assert(address.errors.invalid?(:state_or_province))
    assert(!address.save)
  end

  def test_requires_postal_code
    address = Address.new
    assert(!address.valid?)
    assert(address.errors.invalid?(:postal_code))
    assert(!address.save)
  end

  def test_all_requires
    address = Address.new
    assert(!address.valid?)
    assert(address.errors.invalid?(:street_1))
    address.street_1 = addresses(:one).street_1
    assert(!address.valid?)
    assert(!address.errors.invalid?(:street_1))
    assert(address.errors.invalid?(:city))
    address.city = addresses(:one).city
    assert(!address.valid?)
    assert(!address.errors.invalid?(:city))
    assert(address.errors.invalid?(:state_or_province))
    address.state_or_province = addresses(:one).state_or_province
    assert(!address.valid?)
    assert(!address.errors.invalid?(:state_or_province))
    assert(address.errors.invalid?(:postal_code))
    address.postal_code = addresses(:one).postal_code
    assert(address.valid?)
    assert(!address.errors.invalid?(:postal_code))
    assert(address.save)
  end

end
