require 'test_helper'

class ClubTest < ActiveSupport::TestCase
  fixtures :clubs, :addresses, :contact_points, :contact_types

  def test_requires_name
    club = Club.new
    assert(!club.valid?)
    assert(club.errors.invalid?(:name))
    club.name = clubs(:one).name
    assert(!club.valid?)
    assert(!club.errors.invalid?(:name))
    assert(!club.save)
  end

  def test_requires_subdomain
    club = Club.new
    assert(!club.valid?)
    assert(club.errors.invalid?(:subdomain))
    club.name = clubs(:one).subdomain
    assert(!club.valid?)
    assert(!club.errors.invalid?(:subdomain))
    assert(!club.save)
  end

  def test_requires_address
    club = Club.new
    assert(!club.valid?)
    assert(club.errors.invalid?(:address))
    club.add_address( addresses(:one).attributes )
    assert(!club.valid?)
    assert(!club.errors.invalid?(:address))
    assert(!club.save)
  end

  def test_requires_primary_contact
    club = Club.new
    assert(!club.valid?)
    assert(club.errors.invalid?(:primary_contact))
    contact_point = ContactPoint.new( contact_points(:phone).attributes )
    contact_point.contact_type = ContactType(:phone)
    club.add_primary_contact( contact_point )
    assert(!club.valid?)
    assert(!club.errors.invalid?(:primary_contact))
    assert(!club.save)
  end

  def test_all_requires
    club = Club.new
    assert(!club.valid?)
    assert(club.errors.invalid?(:name))
    club.name = "#{clubs(:one).name}#{clubs(:one).name}"
    assert(!club.valid?)
    assert(!club.errors.invalid?(:name))
    assert(club.errors.invalid?(:subdomain))
    club.name = clubs(:one).subdomain.reverse
    assert(!club.valid?)
    assert(!club.errors.invalid?(:subdomain))
    assert(club.errors.invalid?(:address))
    club.add_address( addresses(:one).attributes )
    assert(!club.valid?)
    assert(!club.errors.invalid?(:address))
    assert(club.errors.invalid?(:primary_contact))
    contact_point = ContactPoint.new( contact_points(:phone).attributes )
    contact_point.contact_type = ContactType(:phone)
    club.add_primary_contact( contact_point )
    assert(club.valid?)
    assert(!club.errors.invalid?(:primary_contact))
    assert(club.save)
  end

  def test_new_club_creation
    club = Club.new_club(
      clubs(:two).attributes,
      addresses(:one).attributes,
      [ contact_points(:email).attributes, contact_points(:phone).attributes ]
    )
    assert(!club.valid?)
    assert(club.errors.invalid?(:name))
    assert(club.errors.invalid?(:subdomain))
    club.name = "#{club.name}#{club.subdomain}"
    club.subdomain = "#{club.subdomain}#{club.subdomain}"
    assert(club.valid?)
    assert(club.save)
  end
  
end
