require 'test_helper'

class ContactPointsTest < ActiveSupport::TestCase
  fixtures :contact_types, :contact_points

  def test_requires_point
    contact_point = ContactPoint.new
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    assert(!contact_point.save)
  end

  def test_requires_contact_type
    contact_point = ContactPoint.new
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:contact_type))
    assert(!contact_point.save)
  end

  def test_all_requires
    contact_point = ContactPoint.new
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:contact_type))
    contact_point.contact_type = contact_types(:email)
    assert(!contact_point.valid?)
    assert(!contact_point.errors.invalid?(:contact_type))
    assert(contact_point.errors.invalid?(:point))
    contact_point.point = contact_points(:email).point
    assert(contact_point.valid?)
    assert(!contact_point.errors.invalid?(:point))
    assert(contact_point.save)
  end
  
  def test_email_point_format
    contact_point = ContactPoint.new
    contact_point.contact_type = contact_types(:email)
    contact_point.point = "email"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    contact_point.point = "email@"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    contact_point.point = "email@domain"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    contact_point.point = "email@domain.com"
    assert(contact_point.valid?)
    assert(!contact_point.errors.invalid?(:point))
    assert(contact_point.save)
  end
  
  def test_phone_point_format
    contact_point = ContactPoint.new
    contact_point.contact_type = contact_types(:phone)
    
    contact_point.point = "2"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    
    contact_point.point = "21"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    
    contact_point.point = "213"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    
    contact_point.point = "2134"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    
    contact_point.point = "21345"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    
    contact_point.point = "213456"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    
    contact_point.point = "2134567"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    
    contact_point.point = "2134-567"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    
    contact_point.point = "213-4576"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    
    contact_point.point = "12134567"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    
    contact_point.point = "122134567"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))
    
    contact_point.point = "2132134567"
    assert(contact_point.valid?)
    assert(!contact_point.errors.invalid?(:point))

    contact_point.point = "12132134567"
    assert(contact_point.valid?)
    assert(!contact_point.errors.invalid?(:point))

    contact_point.point = "1-1-23213-4567"
    assert(!contact_point.valid?)
    assert(contact_point.errors.invalid?(:point))

    contact_point.point = "1-213-213-4567"
    assert(contact_point.valid?)
    assert(!contact_point.errors.invalid?(:point))

    assert(contact_point.save)
  end

end
